import 'dart:async';

import 'package:flutter/material.dart';
import 'package:webview_flutter/webview_flutter.dart';

import '../model/java_script_channel.dart';
import '../widgets/custom_webview.dart';

class CustomController extends ChangeNotifier {
  CustomController.loadRequest(
      {required Uri requestedUrl, String? title, TextStyle? appBarTextStyle}) {
    _controller = WebViewController();
    _title = title;
    _appBarTextStyle = appBarTextStyle;
    _controller.loadRequest(requestedUrl);
  }

  CustomController.loadFile(
      {required String html,
      String? baseUrl,
      String? title,
      TextStyle? appBarTextStyle}) {
    _controller = WebViewController();
    _title = title;
    _appBarTextStyle = appBarTextStyle;
    _controller.loadHtmlString(html, baseUrl: baseUrl);
  }

  CustomController.loadHtmlString({
    required String absoluteFilePath,
    String? title,
    TextStyle? appBarTextStyle,
  }) {
    _controller = WebViewController();
    _title = title;
    _appBarTextStyle = appBarTextStyle;
    _controller.loadFile(absoluteFilePath);
  }

  /// 内部访问WebView的接口
  late CustomWebViewState _customWebViewState;
  void bindState(CustomWebViewState state) {
    _customWebViewState = state;
  }

  /// 外部访问WebViewController的接口
  late WebViewController _controller;
  WebViewController get controller => _controller;

  /// 内部访问AppBar的title接口
  String? _title;
  AppBar? get appBar => _title == null
      ? null
      : AppBar(
          title: ListenableBuilder(
            listenable: this,
            builder: (context, child) {
              return Text(_title!, style: _appBarTextStyle);
            },
          ),
        );
  Future<bool> setTitle(String title) async {
    if (!_customWebViewState.mounted) {
      return Future.value(false);
    } else {
      _title = title;
      notifyListeners();
      return Future.value(true);
    }
  }

  /// 外部访问AppBar的title样式接口
  TextStyle? _appBarTextStyle;
  TextStyle? get appBarTextStyle => _appBarTextStyle;
  Future<bool> setAppBarTextStyle(TextStyle? style) async {
    if (!_customWebViewState.mounted) {
      return Future.value(false);
    } else {
      _appBarTextStyle = style;
      notifyListeners();
      return Future.value(true);
    }
  }

  /// 外部访问JavaScriptChannel的接口
  final List<JavaScriptChannel> _javaScriptChannels = [];
  List<JavaScriptChannel>? get javaScriptChannels => _javaScriptChannels;
  Future<bool> setJavaScriptChannels(JavaScriptChannel channel) async {
    if (_javaScriptChannels
        .where((element) => element.name == channel.name)
        .isNotEmpty) {
      return Future.value(false);
    }
    _javaScriptChannels.add(channel);
    controller.addJavaScriptChannel(channel.name,
        onMessageReceived: channel.channelCallback.run);
    controller.reload();
    return Future.value(true);
  }
}
